<?php

namespace Drp\UserManagement\Policies;

use Drp\UserManagement\Contracts\ManagedUser;
use Illuminate\Auth\Access\HandlesAuthorization;

/**
 * Class UsersPolicy
 * @package App\Policies
 */
class UsersPolicy
{
    use HandlesAuthorization;

    /**
     * Determine if the given user can manage admins.
     *
     * @param  ManagedUser $user The logged in user
     * @return boolean
     */
    public function manage(ManagedUser $user)
    {
        return $user->canManageAdmins();
    }

    /**
     * Determine if the given user can manage admins.
     *
     * @param  ManagedUser $user The logged in user
     * @param  mixed $userToUpdate The user who's role is being updated
     * @return boolean
     */
    public function changeRole(ManagedUser $user, $userToUpdate)
    {
        return $user->id !== $userToUpdate->id;
    }

    /**
     * Determine if the given user can delete the given user.
     *
     * @param ManagedUser $currentUser The logged in user
     * @param ManagedUser $userToDelete The user being deleted
     * @return boolean
     */
    public function delete(ManagedUser $currentUser, ManagedUser $userToDelete)
    {
        return $currentUser->id !== $userToDelete->id;
    }
}
